# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import json
import sys
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, TypeVar, Union, overload

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._model_base import AzureJSONEncoder, _deserialize
from ..._operations._operations import (
    build_anomaly_detector_delete_multivariate_model_request,
    build_anomaly_detector_detect_multivariate_batch_anomaly_request,
    build_anomaly_detector_detect_multivariate_last_anomaly_request,
    build_anomaly_detector_detect_univariate_change_point_request,
    build_anomaly_detector_detect_univariate_entire_series_request,
    build_anomaly_detector_detect_univariate_last_point_request,
    build_anomaly_detector_get_multivariate_batch_detection_result_request,
    build_anomaly_detector_get_multivariate_model_request,
    build_anomaly_detector_list_multivariate_models_request,
    build_anomaly_detector_train_multivariate_model_request,
)
from .._vendor import AnomalyDetectorClientMixinABC

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class AnomalyDetectorClientOperationsMixin(AnomalyDetectorClientMixinABC):
    @overload
    async def detect_univariate_entire_series(
        self, options: _models.UnivariateDetectionOptions, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateEntireDetectionResult:
        """Detect anomalies for the entire series in batch.

        This operation generates a model with an entire series, each point is detected
        with the same model. With this method, points before and after a certain point
        are used to determine whether it is an anomaly. The entire detection can give
        user an overall status of the time series.

        :param options: Method of univariate anomaly detection. Required.
        :type options: ~anomalydetector.models.UnivariateDetectionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateEntireDetectionResult. The UnivariateEntireDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateEntireDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_entire_series(
        self, options: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateEntireDetectionResult:
        """Detect anomalies for the entire series in batch.

        This operation generates a model with an entire series, each point is detected
        with the same model. With this method, points before and after a certain point
        are used to determine whether it is an anomaly. The entire detection can give
        user an overall status of the time series.

        :param options: Method of univariate anomaly detection. Required.
        :type options: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateEntireDetectionResult. The UnivariateEntireDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateEntireDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_entire_series(
        self, options: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateEntireDetectionResult:
        """Detect anomalies for the entire series in batch.

        This operation generates a model with an entire series, each point is detected
        with the same model. With this method, points before and after a certain point
        are used to determine whether it is an anomaly. The entire detection can give
        user an overall status of the time series.

        :param options: Method of univariate anomaly detection. Required.
        :type options: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateEntireDetectionResult. The UnivariateEntireDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateEntireDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def detect_univariate_entire_series(
        self, options: Union[_models.UnivariateDetectionOptions, JSON, IO], **kwargs: Any
    ) -> _models.UnivariateEntireDetectionResult:
        """Detect anomalies for the entire series in batch.

        This operation generates a model with an entire series, each point is detected
        with the same model. With this method, points before and after a certain point
        are used to determine whether it is an anomaly. The entire detection can give
        user an overall status of the time series.

        :param options: Method of univariate anomaly detection. Is one of the following types: model,
         JSON, IO Required.
        :type options: ~anomalydetector.models.UnivariateDetectionOptions or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: UnivariateEntireDetectionResult. The UnivariateEntireDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateEntireDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.UnivariateEntireDetectionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(options, (IO, bytes)):
            _content = options
        else:
            _content = json.dumps(options, cls=AzureJSONEncoder)

        request = build_anomaly_detector_detect_univariate_entire_series_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.AnomalyDetectorError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.UnivariateEntireDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def detect_univariate_last_point(
        self, options: _models.UnivariateDetectionOptions, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateLastDetectionResult:
        """Detect anomaly status of the latest point in time series.

        This operation generates a model using the points that you sent into the API,
        and based on all data to determine whether the last point is anomalous.

        :param options: Method of univariate anomaly detection. Required.
        :type options: ~anomalydetector.models.UnivariateDetectionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateLastDetectionResult. The UnivariateLastDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.UnivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_last_point(
        self, options: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateLastDetectionResult:
        """Detect anomaly status of the latest point in time series.

        This operation generates a model using the points that you sent into the API,
        and based on all data to determine whether the last point is anomalous.

        :param options: Method of univariate anomaly detection. Required.
        :type options: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateLastDetectionResult. The UnivariateLastDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.UnivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_last_point(
        self, options: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateLastDetectionResult:
        """Detect anomaly status of the latest point in time series.

        This operation generates a model using the points that you sent into the API,
        and based on all data to determine whether the last point is anomalous.

        :param options: Method of univariate anomaly detection. Required.
        :type options: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateLastDetectionResult. The UnivariateLastDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.UnivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def detect_univariate_last_point(
        self, options: Union[_models.UnivariateDetectionOptions, JSON, IO], **kwargs: Any
    ) -> _models.UnivariateLastDetectionResult:
        """Detect anomaly status of the latest point in time series.

        This operation generates a model using the points that you sent into the API,
        and based on all data to determine whether the last point is anomalous.

        :param options: Method of univariate anomaly detection. Is one of the following types: model,
         JSON, IO Required.
        :type options: ~anomalydetector.models.UnivariateDetectionOptions or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: UnivariateLastDetectionResult. The UnivariateLastDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.UnivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.UnivariateLastDetectionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(options, (IO, bytes)):
            _content = options
        else:
            _content = json.dumps(options, cls=AzureJSONEncoder)

        request = build_anomaly_detector_detect_univariate_last_point_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.AnomalyDetectorError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.UnivariateLastDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def detect_univariate_change_point(
        self,
        options: _models.UnivariateChangePointDetectionOptions,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.UnivariateChangePointDetectionResult:
        """Detect change point for the entire series.

        Evaluate change point score of every series point.

        :param options: Method of univariate anomaly detection. Required.
        :type options: ~anomalydetector.models.UnivariateChangePointDetectionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateChangePointDetectionResult. The UnivariateChangePointDetectionResult is
         compatible with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateChangePointDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_change_point(
        self, options: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateChangePointDetectionResult:
        """Detect change point for the entire series.

        Evaluate change point score of every series point.

        :param options: Method of univariate anomaly detection. Required.
        :type options: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateChangePointDetectionResult. The UnivariateChangePointDetectionResult is
         compatible with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateChangePointDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_univariate_change_point(
        self, options: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.UnivariateChangePointDetectionResult:
        """Detect change point for the entire series.

        Evaluate change point score of every series point.

        :param options: Method of univariate anomaly detection. Required.
        :type options: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnivariateChangePointDetectionResult. The UnivariateChangePointDetectionResult is
         compatible with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateChangePointDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def detect_univariate_change_point(
        self, options: Union[_models.UnivariateChangePointDetectionOptions, JSON, IO], **kwargs: Any
    ) -> _models.UnivariateChangePointDetectionResult:
        """Detect change point for the entire series.

        Evaluate change point score of every series point.

        :param options: Method of univariate anomaly detection. Is one of the following types: model,
         JSON, IO Required.
        :type options: ~anomalydetector.models.UnivariateChangePointDetectionOptions or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: UnivariateChangePointDetectionResult. The UnivariateChangePointDetectionResult is
         compatible with MutableMapping
        :rtype: ~anomalydetector.models.UnivariateChangePointDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.UnivariateChangePointDetectionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(options, (IO, bytes)):
            _content = options
        else:
            _content = json.dumps(options, cls=AzureJSONEncoder)

        request = build_anomaly_detector_detect_univariate_change_point_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.AnomalyDetectorError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.UnivariateChangePointDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def get_multivariate_batch_detection_result(
        self, result_id: str, **kwargs: Any
    ) -> _models.MultivariateDetectionResult:
        """Get Multivariate Anomaly Detection Result.

        For asynchronous inference, get multivariate anomaly detection result based on
        resultId returned by the BatchDetectAnomaly api.

        :param result_id: ID of a batch detection result. Required.
        :type result_id: str
        :return: MultivariateDetectionResult. The MultivariateDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.MultivariateDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.MultivariateDetectionResult] = kwargs.pop("cls", None)

        request = build_anomaly_detector_get_multivariate_batch_detection_result_request(
            result_id=result_id,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.MultivariateDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def train_multivariate_model(
        self, model_info: _models.ModelInfo, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnomalyDetectionModel:
        """Train a Multivariate Anomaly Detection Model.

        Create and train a multivariate anomaly detection model. The request must
        include a source parameter to indicate an externally accessible Azure blob
        storage URI.There are two types of data input: An URI pointed to an Azure blob
        storage folder which contains multiple CSV files, and each CSV file contains
        two columns, timestamp and variable. Another type of input is an URI pointed to
        a CSV file in Azure blob storage, which contains all the variables and a
        timestamp column.

        :param model_info: Model information. Required.
        :type model_info: ~anomalydetector.models.ModelInfo
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnomalyDetectionModel. The AnomalyDetectionModel is compatible with MutableMapping
        :rtype: ~anomalydetector.models.AnomalyDetectionModel
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def train_multivariate_model(
        self, model_info: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnomalyDetectionModel:
        """Train a Multivariate Anomaly Detection Model.

        Create and train a multivariate anomaly detection model. The request must
        include a source parameter to indicate an externally accessible Azure blob
        storage URI.There are two types of data input: An URI pointed to an Azure blob
        storage folder which contains multiple CSV files, and each CSV file contains
        two columns, timestamp and variable. Another type of input is an URI pointed to
        a CSV file in Azure blob storage, which contains all the variables and a
        timestamp column.

        :param model_info: Model information. Required.
        :type model_info: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnomalyDetectionModel. The AnomalyDetectionModel is compatible with MutableMapping
        :rtype: ~anomalydetector.models.AnomalyDetectionModel
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def train_multivariate_model(
        self, model_info: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.AnomalyDetectionModel:
        """Train a Multivariate Anomaly Detection Model.

        Create and train a multivariate anomaly detection model. The request must
        include a source parameter to indicate an externally accessible Azure blob
        storage URI.There are two types of data input: An URI pointed to an Azure blob
        storage folder which contains multiple CSV files, and each CSV file contains
        two columns, timestamp and variable. Another type of input is an URI pointed to
        a CSV file in Azure blob storage, which contains all the variables and a
        timestamp column.

        :param model_info: Model information. Required.
        :type model_info: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: AnomalyDetectionModel. The AnomalyDetectionModel is compatible with MutableMapping
        :rtype: ~anomalydetector.models.AnomalyDetectionModel
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def train_multivariate_model(
        self, model_info: Union[_models.ModelInfo, JSON, IO], **kwargs: Any
    ) -> _models.AnomalyDetectionModel:
        """Train a Multivariate Anomaly Detection Model.

        Create and train a multivariate anomaly detection model. The request must
        include a source parameter to indicate an externally accessible Azure blob
        storage URI.There are two types of data input: An URI pointed to an Azure blob
        storage folder which contains multiple CSV files, and each CSV file contains
        two columns, timestamp and variable. Another type of input is an URI pointed to
        a CSV file in Azure blob storage, which contains all the variables and a
        timestamp column.

        :param model_info: Model information. Is one of the following types: model, JSON, IO Required.
        :type model_info: ~anomalydetector.models.ModelInfo or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: AnomalyDetectionModel. The AnomalyDetectionModel is compatible with MutableMapping
        :rtype: ~anomalydetector.models.AnomalyDetectionModel
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.AnomalyDetectionModel] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(model_info, (IO, bytes)):
            _content = model_info
        else:
            _content = json.dumps(model_info, cls=AzureJSONEncoder)

        request = build_anomaly_detector_train_multivariate_model_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["location"] = self._deserialize("str", response.headers.get("location"))

        deserialized = _deserialize(_models.AnomalyDetectionModel, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def list_multivariate_models(
        self, *, skip: Optional[int] = None, top: Optional[int] = None, **kwargs: Any
    ) -> AsyncIterable["_models.AnomalyDetectionModel"]:
        """List Multivariate Models.

        List models of a resource.

        :keyword skip: Skip indicates how many models will be skipped. Default value is None.
        :paramtype skip: int
        :keyword top: Top indicates how many models will be fetched. Default value is None.
        :paramtype top: int
        :return: An iterator like instance of AnomalyDetectionModel. The AnomalyDetectionModel is
         compatible with MutableMapping
        :rtype: ~azure.core.async_paging.AsyncItemPaged[~anomalydetector.models.AnomalyDetectionModel]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models._models.ModelList] = kwargs.pop("cls", None)  # pylint: disable=protected-access

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_anomaly_detector_list_multivariate_models_request(
                    skip=skip,
                    top=top,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "Endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                    "ApiVersion": self._serialize.url(
                        "self._config.api_version", self._config.api_version, "str", skip_quote=True
                    ),
                }
                request.url = self._client.format_url(request.url, **path_format_arguments)

            else:
                request = HttpRequest("GET", next_link)
                path_format_arguments = {
                    "Endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                    "ApiVersion": self._serialize.url(
                        "self._config.api_version", self._config.api_version, "str", skip_quote=True
                    ),
                }
                request.url = self._client.format_url(request.url, **path_format_arguments)

            return request

        async def extract_data(pipeline_response):
            deserialized = _deserialize(_models._models.ModelList, pipeline_response)
            list_of_elem = deserialized.models
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
                request, stream=False, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = _deserialize(_models.ResponseError, response.json())
                raise HttpResponseError(response=response, model=error)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def delete_multivariate_model(  # pylint: disable=inconsistent-return-statements
        self, model_id: str, **kwargs: Any
    ) -> None:
        """Delete Multivariate Model.

        Delete an existing multivariate model according to the modelId.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_anomaly_detector_delete_multivariate_model_request(
            model_id=model_id,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})

    @distributed_trace_async
    async def get_multivariate_model(self, model_id: str, **kwargs: Any) -> _models.AnomalyDetectionModel:
        """Get Multivariate Model.

        Get detailed information of multivariate model, including the training status
        and variables used in the model.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :return: AnomalyDetectionModel. The AnomalyDetectionModel is compatible with MutableMapping
        :rtype: ~anomalydetector.models.AnomalyDetectionModel
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[_models.AnomalyDetectionModel] = kwargs.pop("cls", None)

        request = build_anomaly_detector_get_multivariate_model_request(
            model_id=model_id,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.AnomalyDetectionModel, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def detect_multivariate_batch_anomaly(
        self,
        model_id: str,
        options: _models.MultivariateBatchDetectionOptions,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MultivariateDetectionResult:
        """Detect Multivariate Anomaly.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, the input schema should be the same with the training
        request. The request will complete asynchronously and return a resultId to
        query the detection result.The request should be a source link to indicate an
        externally accessible Azure storage Uri, either pointed to an Azure blob
        storage folder, or pointed to a CSV file in Azure blob storage.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of multivariate anomaly detection. Required.
        :type options: ~anomalydetector.models.MultivariateBatchDetectionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateDetectionResult. The MultivariateDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.MultivariateDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_multivariate_batch_anomaly(
        self, model_id: str, options: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MultivariateDetectionResult:
        """Detect Multivariate Anomaly.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, the input schema should be the same with the training
        request. The request will complete asynchronously and return a resultId to
        query the detection result.The request should be a source link to indicate an
        externally accessible Azure storage Uri, either pointed to an Azure blob
        storage folder, or pointed to a CSV file in Azure blob storage.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of multivariate anomaly detection. Required.
        :type options: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateDetectionResult. The MultivariateDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.MultivariateDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_multivariate_batch_anomaly(
        self, model_id: str, options: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MultivariateDetectionResult:
        """Detect Multivariate Anomaly.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, the input schema should be the same with the training
        request. The request will complete asynchronously and return a resultId to
        query the detection result.The request should be a source link to indicate an
        externally accessible Azure storage Uri, either pointed to an Azure blob
        storage folder, or pointed to a CSV file in Azure blob storage.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of multivariate anomaly detection. Required.
        :type options: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateDetectionResult. The MultivariateDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.MultivariateDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def detect_multivariate_batch_anomaly(
        self, model_id: str, options: Union[_models.MultivariateBatchDetectionOptions, JSON, IO], **kwargs: Any
    ) -> _models.MultivariateDetectionResult:
        """Detect Multivariate Anomaly.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, the input schema should be the same with the training
        request. The request will complete asynchronously and return a resultId to
        query the detection result.The request should be a source link to indicate an
        externally accessible Azure storage Uri, either pointed to an Azure blob
        storage folder, or pointed to a CSV file in Azure blob storage.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of multivariate anomaly detection. Is one of the following types:
         model, JSON, IO Required.
        :type options: ~anomalydetector.models.MultivariateBatchDetectionOptions or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: MultivariateDetectionResult. The MultivariateDetectionResult is compatible with
         MutableMapping
        :rtype: ~anomalydetector.models.MultivariateDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.MultivariateDetectionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(options, (IO, bytes)):
            _content = options
        else:
            _content = json.dumps(options, cls=AzureJSONEncoder)

        request = build_anomaly_detector_detect_multivariate_batch_anomaly_request(
            model_id=model_id,
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        response_headers = {}
        response_headers["operation-id"] = self._deserialize("str", response.headers.get("operation-id"))
        response_headers["operation-location"] = self._deserialize("str", response.headers.get("operation-location"))

        deserialized = _deserialize(_models.MultivariateDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def detect_multivariate_last_anomaly(
        self,
        model_id: str,
        options: _models.MultivariateLastDetectionOptions,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.MultivariateLastDetectionResult:
        """Detect anomalies in the last point of the request body.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, and the inference data should be put into request body in a
        JSON format. The request will complete synchronously and return the detection
        immediately in the response body.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of last detection. Required.
        :type options: ~anomalydetector.models.MultivariateLastDetectionOptions
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateLastDetectionResult. The MultivariateLastDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.MultivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_multivariate_last_anomaly(
        self, model_id: str, options: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MultivariateLastDetectionResult:
        """Detect anomalies in the last point of the request body.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, and the inference data should be put into request body in a
        JSON format. The request will complete synchronously and return the detection
        immediately in the response body.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of last detection. Required.
        :type options: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateLastDetectionResult. The MultivariateLastDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.MultivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def detect_multivariate_last_anomaly(
        self, model_id: str, options: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.MultivariateLastDetectionResult:
        """Detect anomalies in the last point of the request body.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, and the inference data should be put into request body in a
        JSON format. The request will complete synchronously and return the detection
        immediately in the response body.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of last detection. Required.
        :type options: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: MultivariateLastDetectionResult. The MultivariateLastDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.MultivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def detect_multivariate_last_anomaly(
        self, model_id: str, options: Union[_models.MultivariateLastDetectionOptions, JSON, IO], **kwargs: Any
    ) -> _models.MultivariateLastDetectionResult:
        """Detect anomalies in the last point of the request body.

        Submit multivariate anomaly detection task with the modelId of trained model
        and inference data, and the inference data should be put into request body in a
        JSON format. The request will complete synchronously and return the detection
        immediately in the response body.

        :param model_id: Model identifier. Required.
        :type model_id: str
        :param options: Request of last detection. Is one of the following types: model, JSON, IO
         Required.
        :type options: ~anomalydetector.models.MultivariateLastDetectionOptions or JSON or IO
        :keyword content_type: Body parameter Content-Type. Known values are: application/json. Default
         value is None.
        :paramtype content_type: str
        :return: MultivariateLastDetectionResult. The MultivariateLastDetectionResult is compatible
         with MutableMapping
        :rtype: ~anomalydetector.models.MultivariateLastDetectionResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.MultivariateLastDetectionResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(options, (IO, bytes)):
            _content = options
        else:
            _content = json.dumps(options, cls=AzureJSONEncoder)

        request = build_anomaly_detector_detect_multivariate_last_anomaly_request(
            model_id=model_id,
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            "ApiVersion": self._serialize.url(
                "self._config.api_version", self._config.api_version, "str", skip_quote=True
            ),
        }
        request.url = self._client.format_url(request.url, **path_format_arguments)

        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # type: ignore # pylint: disable=protected-access
            request, stream=False, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = _deserialize(_models.ResponseError, response.json())
            raise HttpResponseError(response=response, model=error)

        deserialized = _deserialize(_models.MultivariateLastDetectionResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
import sys
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, Type, TypeVar, Union, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._dpp_resource_guard_proxy_operations import (
    build_create_or_update_request,
    build_delete_request,
    build_get_request,
    build_list_request,
    build_unlock_delete_request,
)

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class DppResourceGuardProxyOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.dataprotection.aio.DataProtectionMgmtClient`'s
        :attr:`dpp_resource_guard_proxy` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(
        self, resource_group_name: str, vault_name: str, **kwargs: Any
    ) -> AsyncIterable["_models.ResourceGuardProxyBaseResource"]:
        """Returns the list of ResourceGuardProxies associated with the vault.

        Returns the list of ResourceGuardProxies associated with the vault.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :return: An iterator like instance of either ResourceGuardProxyBaseResource or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ResourceGuardProxyBaseResourceList] = kwargs.pop("cls", None)

        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    resource_group_name=resource_group_name,
                    vault_name=vault_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request = _convert_request(_request)
                _request.url = self._client.format_url(_request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                _request = _convert_request(_request)
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("ResourceGuardProxyBaseResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    @distributed_trace_async
    async def get(
        self, resource_group_name: str, vault_name: str, resource_guard_proxy_name: str, **kwargs: Any
    ) -> _models.ResourceGuardProxyBaseResource:
        """Returns the ResourceGuardProxy object associated with the vault, and that matches the name in
        the request.

        Returns the ResourceGuardProxy object associated with the vault, and that matches the name in
        the request.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :return: ResourceGuardProxyBaseResource or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.ResourceGuardProxyBaseResource] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_group_name=resource_group_name,
            vault_name=vault_name,
            resource_guard_proxy_name=resource_guard_proxy_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ResourceGuardProxyBaseResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    async def create_or_update(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: _models.ResourceGuardProxyBaseResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ResourceGuardProxyBaseResource:
        """Creates or Updates a ResourceGuardProxy.

        Creates or Updates a ResourceGuardProxy.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Required.
        :type parameters: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ResourceGuardProxyBaseResource or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def create_or_update(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ResourceGuardProxyBaseResource:
        """Creates or Updates a ResourceGuardProxy.

        Creates or Updates a ResourceGuardProxy.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ResourceGuardProxyBaseResource or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def create_or_update(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: Union[_models.ResourceGuardProxyBaseResource, IO[bytes]],
        **kwargs: Any
    ) -> _models.ResourceGuardProxyBaseResource:
        """Creates or Updates a ResourceGuardProxy.

        Creates or Updates a ResourceGuardProxy.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Is either a ResourceGuardProxyBaseResource type
         or a IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource or IO[bytes]
        :return: ResourceGuardProxyBaseResource or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.ResourceGuardProxyBaseResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ResourceGuardProxyBaseResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "ResourceGuardProxyBaseResource")

        _request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            vault_name=vault_name,
            resource_guard_proxy_name=resource_guard_proxy_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ResourceGuardProxyBaseResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def delete(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, vault_name: str, resource_guard_proxy_name: str, **kwargs: Any
    ) -> None:
        """Deletes the ResourceGuardProxy.

        Deletes the ResourceGuardProxy.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            vault_name=vault_name,
            resource_guard_proxy_name=resource_guard_proxy_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @overload
    async def unlock_delete(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: _models.UnlockDeleteRequest,
        x_ms_authorization_auxiliary: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.UnlockDeleteResponse:
        """UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Required.
        :type parameters: ~azure.mgmt.dataprotection.models.UnlockDeleteRequest
        :param x_ms_authorization_auxiliary: Default value is None.
        :type x_ms_authorization_auxiliary: str
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnlockDeleteResponse or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.UnlockDeleteResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def unlock_delete(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: IO[bytes],
        x_ms_authorization_auxiliary: Optional[str] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.UnlockDeleteResponse:
        """UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Required.
        :type parameters: IO[bytes]
        :param x_ms_authorization_auxiliary: Default value is None.
        :type x_ms_authorization_auxiliary: str
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: UnlockDeleteResponse or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.UnlockDeleteResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def unlock_delete(
        self,
        resource_group_name: str,
        vault_name: str,
        resource_guard_proxy_name: str,
        parameters: Union[_models.UnlockDeleteRequest, IO[bytes]],
        x_ms_authorization_auxiliary: Optional[str] = None,
        **kwargs: Any
    ) -> _models.UnlockDeleteResponse:
        """UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        UnlockDelete call for ResourceGuardProxy, executed before one can delete it.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :param resource_guard_proxy_name: name of the resource guard proxy. Required.
        :type resource_guard_proxy_name: str
        :param parameters: Request body for operation. Is either a UnlockDeleteRequest type or a
         IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.dataprotection.models.UnlockDeleteRequest or IO[bytes]
        :param x_ms_authorization_auxiliary: Default value is None.
        :type x_ms_authorization_auxiliary: str
        :return: UnlockDeleteResponse or the result of cls(response)
        :rtype: ~azure.mgmt.dataprotection.models.UnlockDeleteResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.UnlockDeleteResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "UnlockDeleteRequest")

        _request = build_unlock_delete_request(
            resource_group_name=resource_group_name,
            vault_name=vault_name,
            resource_guard_proxy_name=resource_guard_proxy_name,
            subscription_id=self._config.subscription_id,
            x_ms_authorization_auxiliary=x_ms_authorization_auxiliary,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("UnlockDeleteResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs."""

    INTERNAL = "Internal"


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity that created the resource."""

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
    logs UX. Default value is "user,system".
    """

    USER = "user"
    SYSTEM = "system"
    USER_SYSTEM = "user,system"


class ResourceState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Resources provisioning states."""

    NOT_SPECIFIED = "NotSpecified"
    ACCEPTED = "Accepted"
    CREATING = "Creating"
    SUCCEEDED = "Succeeded"
    FAILED = "Failed"
    CANCELED = "Canceled"
    PROVISIONING_RESOURCES = "ProvisioningResources"
    INSTALLING_APPLICATION = "InstallingApplication"
    CONFIGURING_APPLICATION = "ConfiguringApplication"
    MIGRATING_APPLICATION_DATA = "MigratingApplicationData"
    RUNNING_VALIDATIONS = "RunningValidations"
    CREATING_ARTIFACTS = "CreatingArtifacts"
    DELETING_ARTIFACTS = "DeletingArtifacts"



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

import base64
import json
import calendar
from typing import cast, Tuple, Optional
from datetime import datetime
from azure.core.serialization import TZ_UTC
from azure.core.credentials import AccessToken


def _convert_datetime_to_utc_int(input_datetime) -> int:
    """
    Converts DateTime in local time to the Epoch in UTC in second.

    :param input_datetime: Input datetime
    :type input_datetime: datetime
    :return: Integer
    :rtype: int
    """
    return int(calendar.timegm(input_datetime.utctimetuple()))


def parse_connection_str(conn_str: Optional[str]) -> Tuple[str, str]:
    if conn_str is None:
        raise ValueError("Connection string is undefined.")
    endpoint = None
    shared_access_key = None
    for element in conn_str.split(";"):
        key, _, value = element.partition("=")
        if key.lower() == "endpoint":
            endpoint = value.rstrip("/")
        elif key.lower() == "accesskey":
            shared_access_key = value
    if not all([endpoint, shared_access_key]):
        raise ValueError(
            "Invalid connection string. You can get the connection string from your resource page in the Azure Portal. "
            "The format should be as follows: endpoint=https://<ResourceUrl>/;accesskey=<KeyValue>"
        )
    left_slash_pos = cast(str, endpoint).find("//")
    if left_slash_pos != -1:
        host = cast(str, endpoint)[left_slash_pos + 2 :]
    else:
        host = str(endpoint)

    return host, str(shared_access_key)


def get_current_utc_time() -> str:
    return str(datetime.now(tz=TZ_UTC).strftime("%a, %d %b %Y %H:%M:%S ")) + "GMT"


def get_current_utc_as_int() -> int:
    current_utc_datetime = datetime.utcnow()
    return _convert_datetime_to_utc_int(current_utc_datetime)


def create_access_token(token) -> AccessToken:
    """Creates an instance of azure.core.credentials.AccessToken from a
    string token. The input string is jwt token in the following form:
    <token_header>.<token_payload>.<token_signature>
    This method looks into the token_payload which is a json and extracts the expiry time
    for that token and creates a tuple of type azure.core.credentials.AccessToken
    (<string_token>, <expiry>)
    :param token: User token
    :type token: str
    :return: Instance of azure.core.credentials.AccessToken - token and expiry date of it
    :rtype: ~azure.core.credentials.AccessToken
    """

    token_parse_err_msg = "Token is not formatted correctly"
    parts = token.split(".")

    if len(parts) < 3:
        raise ValueError(token_parse_err_msg)

    try:
        padded_base64_payload = base64.b64decode(parts[1] + "==").decode("ascii")
        payload = json.loads(padded_base64_payload)
        return AccessToken(
            token,
            _convert_datetime_to_utc_int(datetime.fromtimestamp(payload["exp"], TZ_UTC)),
        )
    except ValueError as val_error:
        raise ValueError(token_parse_err_msg) from val_error



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest
from devtools_testutils import (
    add_general_regex_sanitizer,
    add_body_key_sanitizer,
    remove_batch_sanitizers,
    get_credential,
    test_proxy,
)
from azure.ai.agents import AgentsClient
from dotenv import load_dotenv, find_dotenv

if not load_dotenv(find_dotenv(filename="azure_ai_agents_tests.env"), override=True):
    print("Failed to apply environment variables for azure-ai-projects tests.")


class SanitizedValues:
    SUBSCRIPTION_ID = "00000000-0000-0000-0000-000000000000"
    RESOURCE_GROUP_NAME = "00000"
    WORKSPACE_NAME = "00000"
    DATASET_NAME = "00000"
    TENANT_ID = "00000000-0000-0000-0000-000000000000"
    USER_OBJECT_ID = "00000000-0000-0000-0000-000000000000"
    API_KEY = "00000000000000000000000000000000000000000000000000000000000000000000"
    VECTOR_STORE_NAME = "vs_000000000000000000000000"
    # cSpell:disable-next-line
    FILE_BATCH = "vsfb_00000000000000000000000000000000"


@pytest.fixture(scope="session")
def mock_project_scope():
    return {
        "subscription_id": f"{SanitizedValues.SUBSCRIPTION_ID}",
        "resource_group_name": f"{SanitizedValues.RESOURCE_GROUP_NAME}",
        "project_name": f"{SanitizedValues.WORKSPACE_NAME}",
    }


@pytest.fixture(scope="session")
def mock_dataset_name():
    return {
        "dataset_name": f"{SanitizedValues.DATASET_NAME}",
    }


@pytest.fixture(scope="session")
def mock_vector_store_name():
    return {
        "vector_store_name": f"{SanitizedValues.VECTOR_STORE_NAME}",
        "file_batches": f"{SanitizedValues.FILE_BATCH}",
    }


# autouse=True will trigger this fixture on each pytest run, even if it's not explicitly used by a test method
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy):
    return


@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy, mock_project_scope, mock_dataset_name, mock_vector_store_name):

    def azure_workspace_triad_sanitizer():
        """Sanitize subscription, resource group, and workspace."""

        add_general_regex_sanitizer(
            regex=r"/subscriptions/([-\w\._\(\)]+)",
            value=mock_project_scope["subscription_id"],
            group_for_replace="1",
        )

        add_general_regex_sanitizer(
            regex=r"/resource[gG]roups/([-\w\._\(\)]+)",
            value=mock_project_scope["resource_group_name"],
            group_for_replace="1",
        )

        add_general_regex_sanitizer(
            regex=r"/workspaces/([-\w\._\(\)]+)", value=mock_project_scope["project_name"], group_for_replace="1"
        )

        # TODO (Darren): Check why this is needed in addition to the above
        add_general_regex_sanitizer(
            regex=r"%2Fsubscriptions%2F([-\w\._\(\)]+)",
            value=mock_project_scope["subscription_id"],
            group_for_replace="1",
        )

        # TODO (Darren): Check why this is needed in addition to the above
        add_general_regex_sanitizer(
            regex=r"%2Fresource[gG]roups%2F([-\w\._\(\)]+)",
            value=mock_project_scope["resource_group_name"],
            group_for_replace="1",
        )

    azure_workspace_triad_sanitizer()

    add_general_regex_sanitizer(regex=r"/runs/([-\w\._\(\)]+)", value="Sanitized", group_for_replace="1")

    add_general_regex_sanitizer(
        regex=r"/data/([-\w\._\(\)]+)", value=mock_dataset_name["dataset_name"], group_for_replace="1"
    )

    add_general_regex_sanitizer(
        regex=r"/vector_stores/([-\w\._\(\)]+)",
        value=mock_vector_store_name["vector_store_name"],
        group_for_replace="1",
    )

    add_general_regex_sanitizer(
        regex=r"/file_batches/([-\w\._\(\)]+)/", value=mock_vector_store_name["file_batches"], group_for_replace="1"
    )

    # Sanitize Application Insights connection string from service response (/tests/telemetry)
    add_body_key_sanitizer(
        json_path="properties.ConnectionString",
        value="InstrumentationKey=00000000-0000-0000-0000-000000000000;IngestionEndpoint=https://region.applicationinsights.azure.com/;LiveEndpoint=https://region.livediagnostics.monitor.azure.com/;ApplicationId=00000000-0000-0000-0000-000000000000",
    )

    add_body_key_sanitizer(
        json_path="data_sources[*].uri",
        value="azureml://subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/00000/workspaces/00000/datastores/workspaceblobstore/paths/LocalUpload/00000000000/product_info_1.md",
    )

    add_body_key_sanitizer(
        json_path="configuration.data_sources[*].uri",
        value="azureml://subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/00000/workspaces/00000/datastores/workspaceblobstore/paths/LocalUpload/00000000000/product_info_1.md",
    )

    add_body_key_sanitizer(
        json_path="data_source.uri",
        value="azureml://subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/00000/workspaces/00000/datastores/workspaceblobstore/paths/LocalUpload/00000000000/product_info_1.md",
    )

    add_body_key_sanitizer(
        json_path="tool_resources.azure_ai_search.indexes[*].index_connection_id",
        value="/subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/00000/providers/Microsoft.MachineLearningServices/workspaces/00000/connections/someindex",
    )

    # Sanitize API key from service response (/tests/connections)
    add_body_key_sanitizer(json_path="properties.credentials.key", value="Sanitized")

    # Remove the following sanitizers since certain fields are needed in tests and are non-sensitive:
    #  - AZSDK3493: $..name
    #  - AZSDK3430: $..id
    remove_batch_sanitizers(["AZSDK3493"])
    remove_batch_sanitizers(["AZSDK3430"])



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import sys
from typing import Any, Dict, List, Optional, TYPE_CHECKING

from ... import _serialization

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore

if TYPE_CHECKING:
    from .. import models as _models
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object


class ErrorFieldContract(_serialization.Model):
    """Error Field contract.

    :ivar code: Property level error code.
    :vartype code: str
    :ivar message: Human-readable representation of property-level error.
    :vartype message: str
    :ivar target: Property name.
    :vartype target: str
    """

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "target": {"key": "target", "type": "str"},
    }

    def __init__(
        self, *, code: Optional[str] = None, message: Optional[str] = None, target: Optional[str] = None, **kwargs: Any
    ) -> None:
        """
        :keyword code: Property level error code.
        :paramtype code: str
        :keyword message: Human-readable representation of property-level error.
        :paramtype message: str
        :keyword target: Property name.
        :paramtype target: str
        """
        super().__init__(**kwargs)
        self.code = code
        self.message = message
        self.target = target


class WorkbookError(_serialization.Model):
    """Error message body that will indicate why the operation failed.

    :ivar code: Service-defined error code. This code serves as a sub-status for the HTTP error
     code specified in the response.
    :vartype code: str
    :ivar message: Human-readable representation of the error.
    :vartype message: str
    :ivar details: The list of invalid fields send in request, in case of validation error.
    :vartype details:
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.ErrorFieldContract]
    """

    _attribute_map = {
        "code": {"key": "code", "type": "str"},
        "message": {"key": "message", "type": "str"},
        "details": {"key": "details", "type": "[ErrorFieldContract]"},
    }

    def __init__(
        self,
        *,
        code: Optional[str] = None,
        message: Optional[str] = None,
        details: Optional[List["_models.ErrorFieldContract"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword code: Service-defined error code. This code serves as a sub-status for the HTTP error
         code specified in the response.
        :paramtype code: str
        :keyword message: Human-readable representation of the error.
        :paramtype message: str
        :keyword details: The list of invalid fields send in request, in case of validation error.
        :paramtype details:
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.ErrorFieldContract]
        """
        super().__init__(**kwargs)
        self.code = code
        self.message = message
        self.details = details


class WorkbookTemplateResource(_serialization.Model):
    """An azure resource object.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to server.

    :ivar id: Azure resource Id.
    :vartype id: str
    :ivar name: Azure resource name.
    :vartype name: str
    :ivar type: Azure resource type.
    :vartype type: str
    :ivar location: Resource location. Required.
    :vartype location: str
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
        "location": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "location": {"key": "location", "type": "str"},
        "tags": {"key": "tags", "type": "{str}"},
    }

    def __init__(self, *, location: str, tags: Optional[Dict[str, str]] = None, **kwargs: Any) -> None:
        """
        :keyword location: Resource location. Required.
        :paramtype location: str
        :keyword tags: Resource tags.
        :paramtype tags: dict[str, str]
        """
        super().__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None
        self.location = location
        self.tags = tags


class WorkbookTemplate(WorkbookTemplateResource):
    """An Application Insights workbook template definition.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to server.

    :ivar id: Azure resource Id.
    :vartype id: str
    :ivar name: Azure resource name.
    :vartype name: str
    :ivar type: Azure resource type.
    :vartype type: str
    :ivar location: Resource location. Required.
    :vartype location: str
    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar priority: Priority of the template. Determines which template to open when a workbook
     gallery is opened in viewer mode.
    :vartype priority: int
    :ivar author: Information about the author of the workbook template.
    :vartype author: str
    :ivar template_data: Valid JSON object containing workbook template payload.
    :vartype template_data: JSON
    :ivar galleries: Workbook galleries supported by the template.
    :vartype galleries:
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
    :ivar localized: Key value pair of localized gallery. Each key is the locale code of languages
     supported by the Azure portal.
    :vartype localized: dict[str,
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateLocalizedGallery]]
    """

    _validation = {
        "id": {"readonly": True},
        "name": {"readonly": True},
        "type": {"readonly": True},
        "location": {"required": True},
    }

    _attribute_map = {
        "id": {"key": "id", "type": "str"},
        "name": {"key": "name", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "location": {"key": "location", "type": "str"},
        "tags": {"key": "tags", "type": "{str}"},
        "priority": {"key": "properties.priority", "type": "int"},
        "author": {"key": "properties.author", "type": "str"},
        "template_data": {"key": "properties.templateData", "type": "object"},
        "galleries": {"key": "properties.galleries", "type": "[WorkbookTemplateGallery]"},
        "localized": {"key": "properties.localized", "type": "{[WorkbookTemplateLocalizedGallery]}"},
    }

    def __init__(
        self,
        *,
        location: str,
        tags: Optional[Dict[str, str]] = None,
        priority: Optional[int] = None,
        author: Optional[str] = None,
        template_data: Optional[JSON] = None,
        galleries: Optional[List["_models.WorkbookTemplateGallery"]] = None,
        localized: Optional[Dict[str, List["_models.WorkbookTemplateLocalizedGallery"]]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword location: Resource location. Required.
        :paramtype location: str
        :keyword tags: Resource tags.
        :paramtype tags: dict[str, str]
        :keyword priority: Priority of the template. Determines which template to open when a workbook
         gallery is opened in viewer mode.
        :paramtype priority: int
        :keyword author: Information about the author of the workbook template.
        :paramtype author: str
        :keyword template_data: Valid JSON object containing workbook template payload.
        :paramtype template_data: JSON
        :keyword galleries: Workbook galleries supported by the template.
        :paramtype galleries:
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
        :keyword localized: Key value pair of localized gallery. Each key is the locale code of
         languages supported by the Azure portal.
        :paramtype localized: dict[str,
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateLocalizedGallery]]
        """
        super().__init__(location=location, tags=tags, **kwargs)
        self.priority = priority
        self.author = author
        self.template_data = template_data
        self.galleries = galleries
        self.localized = localized


class WorkbookTemplateGallery(_serialization.Model):
    """Gallery information for a workbook template.

    :ivar name: Name of the workbook template in the gallery.
    :vartype name: str
    :ivar category: Category for the gallery.
    :vartype category: str
    :ivar type: Type of workbook supported by the workbook template.
    :vartype type: str
    :ivar order: Order of the template within the gallery.
    :vartype order: int
    :ivar resource_type: Azure resource type supported by the gallery.
    :vartype resource_type: str
    """

    _attribute_map = {
        "name": {"key": "name", "type": "str"},
        "category": {"key": "category", "type": "str"},
        "type": {"key": "type", "type": "str"},
        "order": {"key": "order", "type": "int"},
        "resource_type": {"key": "resourceType", "type": "str"},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        category: Optional[str] = None,
        type: Optional[str] = None,
        order: Optional[int] = None,
        resource_type: Optional[str] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword name: Name of the workbook template in the gallery.
        :paramtype name: str
        :keyword category: Category for the gallery.
        :paramtype category: str
        :keyword type: Type of workbook supported by the workbook template.
        :paramtype type: str
        :keyword order: Order of the template within the gallery.
        :paramtype order: int
        :keyword resource_type: Azure resource type supported by the gallery.
        :paramtype resource_type: str
        """
        super().__init__(**kwargs)
        self.name = name
        self.category = category
        self.type = type
        self.order = order
        self.resource_type = resource_type


class WorkbookTemplateLocalizedGallery(_serialization.Model):
    """Localized template data and gallery information.

    :ivar template_data: Valid JSON object containing workbook template payload.
    :vartype template_data: JSON
    :ivar galleries: Workbook galleries supported by the template.
    :vartype galleries:
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
    """

    _attribute_map = {
        "template_data": {"key": "templateData", "type": "object"},
        "galleries": {"key": "galleries", "type": "[WorkbookTemplateGallery]"},
    }

    def __init__(
        self,
        *,
        template_data: Optional[JSON] = None,
        galleries: Optional[List["_models.WorkbookTemplateGallery"]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword template_data: Valid JSON object containing workbook template payload.
        :paramtype template_data: JSON
        :keyword galleries: Workbook galleries supported by the template.
        :paramtype galleries:
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
        """
        super().__init__(**kwargs)
        self.template_data = template_data
        self.galleries = galleries


class WorkbookTemplatesListResult(_serialization.Model):
    """WorkbookTemplate list result.

    :ivar value: An array of workbook templates.
    :vartype value:
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplate]
    """

    _attribute_map = {
        "value": {"key": "value", "type": "[WorkbookTemplate]"},
    }

    def __init__(self, *, value: Optional[List["_models.WorkbookTemplate"]] = None, **kwargs: Any) -> None:
        """
        :keyword value: An array of workbook templates.
        :paramtype value:
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplate]
        """
        super().__init__(**kwargs)
        self.value = value


class WorkbookTemplateUpdateParameters(_serialization.Model):
    """The parameters that can be provided when updating workbook template.

    :ivar tags: Resource tags.
    :vartype tags: dict[str, str]
    :ivar priority: Priority of the template. Determines which template to open when a workbook
     gallery is opened in viewer mode.
    :vartype priority: int
    :ivar author: Information about the author of the workbook template.
    :vartype author: str
    :ivar template_data: Valid JSON object containing workbook template payload.
    :vartype template_data: JSON
    :ivar galleries: Workbook galleries supported by the template.
    :vartype galleries:
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
    :ivar localized: Key value pair of localized gallery. Each key is the locale code of languages
     supported by the Azure portal.
    :vartype localized: dict[str,
     list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateLocalizedGallery]]
    """

    _attribute_map = {
        "tags": {"key": "tags", "type": "{str}"},
        "priority": {"key": "properties.priority", "type": "int"},
        "author": {"key": "properties.author", "type": "str"},
        "template_data": {"key": "properties.templateData", "type": "object"},
        "galleries": {"key": "properties.galleries", "type": "[WorkbookTemplateGallery]"},
        "localized": {"key": "properties.localized", "type": "{[WorkbookTemplateLocalizedGallery]}"},
    }

    def __init__(
        self,
        *,
        tags: Optional[Dict[str, str]] = None,
        priority: Optional[int] = None,
        author: Optional[str] = None,
        template_data: Optional[JSON] = None,
        galleries: Optional[List["_models.WorkbookTemplateGallery"]] = None,
        localized: Optional[Dict[str, List["_models.WorkbookTemplateLocalizedGallery"]]] = None,
        **kwargs: Any
    ) -> None:
        """
        :keyword tags: Resource tags.
        :paramtype tags: dict[str, str]
        :keyword priority: Priority of the template. Determines which template to open when a workbook
         gallery is opened in viewer mode.
        :paramtype priority: int
        :keyword author: Information about the author of the workbook template.
        :paramtype author: str
        :keyword template_data: Valid JSON object containing workbook template payload.
        :paramtype template_data: JSON
        :keyword galleries: Workbook galleries supported by the template.
        :paramtype galleries:
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateGallery]
        :keyword localized: Key value pair of localized gallery. Each key is the locale code of
         languages supported by the Azure portal.
        :paramtype localized: dict[str,
         list[~azure.mgmt.applicationinsights.v2019_10_17_preview.models.WorkbookTemplateLocalizedGallery]]
        """
        super().__init__(**kwargs)
        self.tags = tags
        self.priority = priority
        self.author = author
        self.template_data = template_data
        self.galleries = galleries
        self.localized = localized



# The MIT License (MIT)
# Copyright (c) 2014 Microsoft Corporation

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Internal class for proxy query execution context implementation in the Azure
Cosmos database service.
"""

import json
import os
from azure.cosmos.exceptions import CosmosHttpResponseError
from azure.cosmos._execution_context import endpoint_component, multi_execution_aggregator
from azure.cosmos._execution_context import non_streaming_order_by_aggregator, hybrid_search_aggregator
from azure.cosmos._execution_context.base_execution_context import _QueryExecutionContextBase
from azure.cosmos._execution_context.base_execution_context import _DefaultQueryExecutionContext
from azure.cosmos._execution_context.query_execution_info import _PartitionedQueryExecutionInfo
from azure.cosmos.documents import _DistinctType
from azure.cosmos.http_constants import StatusCodes, SubStatusCodes
from .._constants import _Constants as Constants

# pylint: disable=protected-access


def _is_partitioned_execution_info(e):
    return (
        e.status_code == StatusCodes.BAD_REQUEST and e.sub_status == SubStatusCodes.CROSS_PARTITION_QUERY_NOT_SERVABLE
    )


def _is_hybrid_search_query(query, e):
    # had to add this logic since error returned from service is different, will need to ask Neil
    if e.status_code == StatusCodes.INTERNAL_SERVER_ERROR:
        if "RRF" in query or "FullTextContains" in query or "FullTextScore" in query:
            return True
    return False


def _verify_valid_hybrid_search_query(hybrid_search_query_info):
    if not hybrid_search_query_info['take']:
        raise ValueError("Executing a hybrid search query without TOP or LIMIT can consume many" +
                         " RUs very fast and have long runtimes. Please ensure you are using one" +
                         " of the two filters with your hybrid search query.")
    if hybrid_search_query_info['take'] > int(os.environ.get(Constants.HS_MAX_ITEMS_CONFIG,
                                                             Constants.HS_MAX_ITEMS_CONFIG_DEFAULT)):
        raise ValueError("Executing a hybrid search query with more items than the max is not allowed. " +
                         "Please ensure you are using a limit smaller than the max, or change the max.")


def _get_partitioned_execution_info(e):
    error_msg = json.loads(e.http_error_message)
    return _PartitionedQueryExecutionInfo(json.loads(error_msg["additionalErrorInfo"]))


class _ProxyQueryExecutionContext(_QueryExecutionContextBase):  # pylint: disable=abstract-method
    """Represents a proxy execution context wrapper.

    By default, uses _DefaultQueryExecutionContext.

    If backend responds a 400 error code with a Query Execution Info, switches
    to _MultiExecutionContextAggregator
    """

    def __init__(self, client, resource_link, query, options, fetch_function, response_hook):
        """
        Constructor
        """
        super(_ProxyQueryExecutionContext, self).__init__(client, options)

        self._execution_context = _DefaultQueryExecutionContext(client, options, fetch_function)
        self._resource_link = resource_link
        self._query = query
        self._fetch_function = fetch_function
        self._response_hook = response_hook

    def __next__(self):
        """Returns the next query result.

        :return: The next query result.
        :rtype: dict
        :raises StopIteration: If no more result is left.

        """
        try:
            return next(self._execution_context)
        except CosmosHttpResponseError as e:
            if _is_partitioned_execution_info(e):
                query_to_use = self._query if self._query is not None else "Select * from root r"
                query_plan_dict = self._client._GetQueryPlanThroughGateway(
                    query_to_use, self._resource_link, self._options.get('excludedLocations'))
                query_execution_info = _PartitionedQueryExecutionInfo(query_plan_dict)
                self._execution_context = self._create_pipelined_execution_context(query_execution_info)
            else:
                raise e

        return next(self._execution_context)

    def fetch_next_block(self):
        """Returns a block of results.

        This method only exists for backward compatibility reasons. (Because
        QueryIterable has exposed fetch_next_block api).

        :return: List of results.
        :rtype: list
        """
        # TODO: NEED to change this - make every query retrieve a query plan
        # also, we can't have this logic being returned to so often - there should be no need for this
        # need to split up query plan logic and actual query iterating logic
        try:
            return self._execution_context.fetch_next_block()
        except CosmosHttpResponseError as e:
            if _is_partitioned_execution_info(e) or _is_hybrid_search_query(self._query, e):
                query_to_use = self._query if self._query is not None else "Select * from root r"
                query_plan_dict = self._client._GetQueryPlanThroughGateway(
                    query_to_use, self._resource_link, self._options.get('excludedLocations'))
                query_execution_info = _PartitionedQueryExecutionInfo(query_plan_dict)
                self._execution_context = self._create_pipelined_execution_context(query_execution_info)
            else:
                raise e

        return self._execution_context.fetch_next_block()

    def _create_pipelined_execution_context(self, query_execution_info):
        assert self._resource_link, "code bug, resource_link is required."
        if query_execution_info.has_aggregates() and not query_execution_info.has_select_value():
            if self._options and ("enableCrossPartitionQuery" in self._options
                                  and self._options["enableCrossPartitionQuery"]):
                raise CosmosHttpResponseError(
                    StatusCodes.BAD_REQUEST,
                    "Cross partition query only supports 'VALUE <AggregateFunc>' for aggregates")

        # throw exception here for vector search query without limit filter or limit > max_limit
        if query_execution_info.get_non_streaming_order_by():
            total_item_buffer = (query_execution_info.get_top() or 0) or \
                                ((query_execution_info.get_limit() or 0) + (query_execution_info.get_offset() or 0))
            if total_item_buffer == 0:
                raise ValueError("Executing a vector search query without TOP or LIMIT can consume many" +
                                 " RUs very fast and have long runtimes. Please ensure you are using one" +
                                 " of the two filters with your vector search query.")
            if total_item_buffer > int(os.environ.get(Constants.MAX_ITEM_BUFFER_VS_CONFIG,
                                                      Constants.MAX_ITEM_BUFFER_VS_CONFIG_DEFAULT)):
                raise ValueError("Executing a vector search query with more items than the max is not allowed. " +
                                 "Please ensure you are using a limit smaller than the max, or change the max.")
            execution_context_aggregator = \
                non_streaming_order_by_aggregator._NonStreamingOrderByContextAggregator(self._client,
                                                                                        self._resource_link,
                                                                                        self._query,
                                                                                        self._options,
                                                                                        query_execution_info,
                                                                                        self._response_hook)
        elif query_execution_info.has_hybrid_search_query_info():
            hybrid_search_query_info = query_execution_info._query_execution_info['hybridSearchQueryInfo']
            _verify_valid_hybrid_search_query(hybrid_search_query_info)
            execution_context_aggregator = \
                hybrid_search_aggregator._HybridSearchContextAggregator(self._client,
                                                                        self._resource_link,
                                                                        self._options,
                                                                        query_execution_info,
                                                                        hybrid_search_query_info,
                                                                        self._response_hook)
            execution_context_aggregator._run_hybrid_search()
        else:
            execution_context_aggregator = \
                multi_execution_aggregator._MultiExecutionContextAggregator(self._client,
                                                                            self._resource_link,
                                                                            self._query,
                                                                            self._options,
                                                                            query_execution_info,
                                                                            self._response_hook)
        return _PipelineExecutionContext(self._client, self._options, execution_context_aggregator,
                                         query_execution_info)

    next = __next__  # Python 2 compatibility.


class _PipelineExecutionContext(_QueryExecutionContextBase):  # pylint: disable=abstract-method

    DEFAULT_PAGE_SIZE = 1000

    def __init__(self, client, options, execution_context, query_execution_info):
        super(_PipelineExecutionContext, self).__init__(client, options)

        if options.get("maxItemCount"):
            self._page_size = options["maxItemCount"]
        else:
            self._page_size = _PipelineExecutionContext.DEFAULT_PAGE_SIZE

        self._execution_context = execution_context

        self._endpoint = endpoint_component._QueryExecutionEndpointComponent(execution_context)

        order_by = query_execution_info.get_order_by()
        if query_execution_info.get_non_streaming_order_by():
            self._endpoint = endpoint_component._QueryExecutionNonStreamingEndpointComponent(self._endpoint)
        elif order_by:
            self._endpoint = endpoint_component._QueryExecutionOrderByEndpointComponent(self._endpoint)

        aggregates = query_execution_info.get_aggregates()
        if aggregates:
            self._endpoint = endpoint_component._QueryExecutionAggregateEndpointComponent(self._endpoint, aggregates)

        distinct_type = query_execution_info.get_distinct_type()
        if distinct_type != _DistinctType.NoneType:
            if distinct_type == _DistinctType.Ordered:
                self._endpoint = endpoint_component._QueryExecutionDistinctOrderedEndpointComponent(self._endpoint)
            else:
                self._endpoint = endpoint_component._QueryExecutionDistinctUnorderedEndpointComponent(self._endpoint)

        offset = query_execution_info.get_offset()
        if offset is not None:
            self._endpoint = endpoint_component._QueryExecutionOffsetEndpointComponent(self._endpoint, offset)

        top = query_execution_info.get_top()
        if top is not None:
            self._endpoint = endpoint_component._QueryExecutionTopEndpointComponent(self._endpoint, top)

        limit = query_execution_info.get_limit()
        if limit is not None:
            self._endpoint = endpoint_component._QueryExecutionTopEndpointComponent(self._endpoint, limit)

    def __next__(self):
        """Returns the next query result.

        :return: The next query result.
        :rtype: dict
        :raises StopIteration: If no more result is left.
        """
        return next(self._endpoint)

    def fetch_next_block(self):
        """Returns a block of results.

        This method only exists for backward compatibility reasons. (Because
        QueryIterable has exposed fetch_next_block api).

        This method internally invokes next() as many times required to collect
        the requested fetch size.

        :return: List of results.
        :rtype: list
        """

        results = []
        for _ in range(self._page_size):
            try:
                results.append(next(self))
            except StopIteration:
                # no more results
                break
        return results

    next = __next__  # Python 2 compatibility.



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is used to verify package dependency by importing all modules
import sys
import argparse
import logging
import os
from subprocess import check_call

from ci_tools.parsing import ParsedSetup

logging.getLogger().setLevel(logging.INFO)
root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "..", ".."))

# keyvault has dependency issue when loading private module _BearerTokenCredentialPolicyBase from azure.core.pipeline.policies
# azure.core.tracing.opencensus and azure.eventhub.checkpointstoreblob.aio are skipped due to a known issue in loading azure.core.tracing.opencensus
excluded_packages = [
    "azure",
    "azure-mgmt",
    ]

def should_run_import_all(package_name):
    return not (package_name in excluded_packages or "nspkg" in package_name)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Import all modules in package")

    parser.add_argument(
        "-t",
        "--target",
        dest="target_package",
        help="The target package directory on disk.",
        required=True,
    )

    args = parser.parse_args()

    # get target package name from target package path
    pkg_dir = os.path.abspath(args.target_package)
    pkg_details = ParsedSetup.from_path(pkg_dir)
    
    if should_run_import_all(pkg_details.name):
        # import all modules from current package
        logging.info(
            "Importing all modules from namespace [{0}] to verify dependency".format(
                pkg_details.namespace
            )
        )
        import_script_all = "from {0} import *".format(pkg_details.namespace)
        commands = [
            sys.executable,
            "-c",
            import_script_all
        ]

        check_call(commands, cwd= root_dir)
        logging.info("Verified module dependency, no issues found")
    else:
        pass
        logging.error("Package {} is excluded from dependency check".format(pkg_details.name))



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, TypeVar, Union

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from ... import models as _models
from ...operations._workspace_notification_recipient_email_operations import (
    build_check_entity_exists_request,
    build_create_or_update_request,
    build_delete_request,
    build_list_by_notification_request,
)

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class WorkspaceNotificationRecipientEmailOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.apimanagement.aio.ApiManagementClient`'s
        :attr:`workspace_notification_recipient_email` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def list_by_notification(
        self,
        resource_group_name: str,
        service_name: str,
        workspace_id: str,
        notification_name: Union[str, _models.NotificationName],
        **kwargs: Any
    ) -> _models.RecipientEmailCollection:
        """Gets the list of the Notification Recipient Emails subscribed to a notification.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :param workspace_id: Workspace identifier. Must be unique in the current API Management service
         instance. Required.
        :type workspace_id: str
        :param notification_name: Notification Name Identifier. Known values are:
         "RequestPublisherNotificationMessage", "PurchasePublisherNotificationMessage",
         "NewApplicationNotificationMessage", "BCC", "NewIssuePublisherNotificationMessage",
         "AccountClosedPublisher", and "QuotaLimitApproachingPublisherNotificationMessage". Required.
        :type notification_name: str or ~azure.mgmt.apimanagement.models.NotificationName
        :return: RecipientEmailCollection or the result of cls(response)
        :rtype: ~azure.mgmt.apimanagement.models.RecipientEmailCollection
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.RecipientEmailCollection] = kwargs.pop("cls", None)

        _request = build_list_by_notification_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            workspace_id=workspace_id,
            notification_name=notification_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RecipientEmailCollection", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def check_entity_exists(
        self,
        resource_group_name: str,
        service_name: str,
        workspace_id: str,
        notification_name: Union[str, _models.NotificationName],
        email: str,
        **kwargs: Any
    ) -> bool:
        """Determine if Notification Recipient Email subscribed to the notification.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :param workspace_id: Workspace identifier. Must be unique in the current API Management service
         instance. Required.
        :type workspace_id: str
        :param notification_name: Notification Name Identifier. Known values are:
         "RequestPublisherNotificationMessage", "PurchasePublisherNotificationMessage",
         "NewApplicationNotificationMessage", "BCC", "NewIssuePublisherNotificationMessage",
         "AccountClosedPublisher", and "QuotaLimitApproachingPublisherNotificationMessage". Required.
        :type notification_name: str or ~azure.mgmt.apimanagement.models.NotificationName
        :param email: Email identifier. Required.
        :type email: str
        :return: bool or the result of cls(response)
        :rtype: bool
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_check_entity_exists_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            workspace_id=workspace_id,
            notification_name=notification_name,
            email=email,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204, 404]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore
        return 200 <= response.status_code <= 299

    @distributed_trace_async
    async def create_or_update(
        self,
        resource_group_name: str,
        service_name: str,
        workspace_id: str,
        notification_name: Union[str, _models.NotificationName],
        email: str,
        **kwargs: Any
    ) -> _models.RecipientEmailContract:
        """Adds the Email address to the list of Recipients for the Notification.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :param workspace_id: Workspace identifier. Must be unique in the current API Management service
         instance. Required.
        :type workspace_id: str
        :param notification_name: Notification Name Identifier. Known values are:
         "RequestPublisherNotificationMessage", "PurchasePublisherNotificationMessage",
         "NewApplicationNotificationMessage", "BCC", "NewIssuePublisherNotificationMessage",
         "AccountClosedPublisher", and "QuotaLimitApproachingPublisherNotificationMessage". Required.
        :type notification_name: str or ~azure.mgmt.apimanagement.models.NotificationName
        :param email: Email identifier. Required.
        :type email: str
        :return: RecipientEmailContract or the result of cls(response)
        :rtype: ~azure.mgmt.apimanagement.models.RecipientEmailContract
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.RecipientEmailContract] = kwargs.pop("cls", None)

        _request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            workspace_id=workspace_id,
            notification_name=notification_name,
            email=email,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("RecipientEmailContract", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def delete(
        self,
        resource_group_name: str,
        service_name: str,
        workspace_id: str,
        notification_name: Union[str, _models.NotificationName],
        email: str,
        **kwargs: Any
    ) -> None:
        """Removes the email from the list of Notification.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param service_name: The name of the API Management service. Required.
        :type service_name: str
        :param workspace_id: Workspace identifier. Must be unique in the current API Management service
         instance. Required.
        :type workspace_id: str
        :param notification_name: Notification Name Identifier. Known values are:
         "RequestPublisherNotificationMessage", "PurchasePublisherNotificationMessage",
         "NewApplicationNotificationMessage", "BCC", "NewIssuePublisherNotificationMessage",
         "AccountClosedPublisher", and "QuotaLimitApproachingPublisherNotificationMessage". Required.
        :type notification_name: str or ~azure.mgmt.apimanagement.models.NotificationName
        :param email: Email identifier. Required.
        :type email: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            service_name=service_name,
            workspace_id=workspace_id,
            notification_name=notification_name,
            email=email,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# pylint: disable=line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
DESCRIPTION:
    This sample demonstrates how to add files to agent during the vector store creation.

USAGE:
    python sample_agents_vector_store_enterprise_file_search_async.py

    Before running the sample:

    pip install azure-ai-agents azure-identity azure-ai-ml aiohttp

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model.
"""
import asyncio
import os

from azure.ai.agents.aio import AgentsClient
from azure.ai.agents.models import (
    FileSearchTool,
    VectorStoreDataSource,
    VectorStoreDataSourceAssetType,
    ListSortOrder,
    MessageTextContent,
)
from azure.identity.aio import DefaultAzureCredential


async def main():
    async with DefaultAzureCredential() as credential:
        async with AgentsClient(
            endpoint=os.environ["PROJECT_ENDPOINT"],
            credential=credential,
        ) as agents_client:
            # We will upload the local file to Azure and will use it for vector store creation.
            asset_uri = os.environ["AZURE_BLOB_URI"]
            ds = VectorStoreDataSource(asset_identifier=asset_uri, asset_type=VectorStoreDataSourceAssetType.URI_ASSET)
            vector_store = await agents_client.vector_stores.create_and_poll(
                data_sources=[ds], name="sample_vector_store"
            )
            print(f"Created vector store, vector store ID: {vector_store.id}")

            # Create a file search tool
            file_search_tool = FileSearchTool(vector_store_ids=[vector_store.id])

            # Notices that FileSearchTool as tool and tool_resources must be added or the agent unable to search the file
            agent = await agents_client.create_agent(
                model=os.environ["MODEL_DEPLOYMENT_NAME"],
                name="my-agent",
                instructions="You are helpful agent",
                tools=file_search_tool.definitions,
                tool_resources=file_search_tool.resources,
            )
            print(f"Created agent, agent ID: {agent.id}")

            thread = await agents_client.threads.create()
            print(f"Created thread, thread ID: {thread.id}")

            message = await agents_client.messages.create(
                thread_id=thread.id, role="user", content="What feature does Smart Eyewear offer?"
            )
            print(f"Created message, message ID: {message.id}")

            run = await agents_client.runs.create_and_process(thread_id=thread.id, agent_id=agent.id)
            print(f"Created run, run ID: {run.id}")

            await agents_client.vector_stores.delete(vector_store.id)
            print("Deleted vector store")

            await agents_client.delete_agent(agent.id)
            print("Deleted agent")

            messages = agents_client.messages.list(thread_id=thread.id, order=ListSortOrder.ASCENDING)
            async for msg in messages:
                last_part = msg.content[-1]
                if isinstance(last_part, MessageTextContent):
                    print(f"{msg.role}: {last_part.text.value}")


if __name__ == "__main__":
    asyncio.run(main())
